import 'package:chessjs/color.dart' as chessColor;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_chessboard_js/chess_vectors/chess_vectors_flutter.dart';
import 'dart:async';

import 'chessboard_controller.dart';

/// A single square on the chessboard
/// Here the state of the chessboard is handle
/// this class receives a string with the name of the square,
/// the chessboard controller and the current size

class BoardSquare extends StatefulWidget {
  BoardSquare({
    required Key key,
    required this.squareName,
    required this.controller,
    required this.size,
  });

  final String squareName;
  final double size;
  final ChessboardController controller;

  @override
  _BoardSquareState createState() => _BoardSquareState();
}

class _BoardSquareState extends State<BoardSquare> {
  @override
  void initState() {
    /// Temporarily workaround to update square after promotion moves
    /// (dont wanna use state management techniques on this, just pure flutter)
    /// The PROBLEM - change notifier/notify listeners update all listeners
    /// I only want to update the square from where the pawn went to promotion

    /// add listener to 2nd/7th row
    if (widget.squareName.contains('2') || widget.squareName.contains('7'))
      widget.controller.addListener(() {
        /// the promotion function will trigger this listener only
        /// if the last move from history == to this [squareName]
        if (widget.controller.gameLogic.history.isNotEmpty) {
          final lastMoveFromSquare = widget.controller.gameLogic.makePretty(
              widget.controller.gameLogic.history.last.move)['from'];

          /// if promotion move came from this [squareName] rebuild widget
          /// to delete the pawn on 2nd/7th rank (PROBLEM is on promotion setState comes
          /// before the dialog and the move is not completed until after the dialog
          if (lastMoveFromSquare == widget.squareName) {
            setState(() {});
          }
        }
      });
    super.initState();
  }

  @override
  void dispose() {
    widget.controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      flex: 1,
      child: DragTarget(builder: (context, accepted, rejected) {
        /// get the piece on the square provided
        return widget.controller.gameLogic.get(widget.squareName) != null
            ?

            /// if there is a piece on the square, returns a draggable image of the piece
            Draggable(
                child: _getImageToDisplay(
                    size: widget.size / 8, game: widget.controller),
                feedback: _getImageToDisplay(
                    size: (1.15 * (widget.size / 8)), game: widget.controller),
                onDragCompleted: () {
                  if (mounted) {
                    setState(() {});
                  }
                },

                /// Data to be dropped in this drag target: the squareName where
                /// the move originated, the type of the piece and the color of the piece
                data: [
                  widget.squareName,
                  widget.controller.gameLogic
                      .get(widget.squareName)!
                      .type
                      .toUpperCase(),
                  widget.controller.gameLogic.get(widget.squareName)!.color,
                ],
              )

            /// Changing here Container for SizedBox wont allow to make a move
            /// Container expands to the preferred size, SizedBox wont, so the drag target
            /// property will be on a 0 size widget and wont work
            : Container();
      }, onWillAccept: (List? moveInfo) {
        /// Accept only if the game turn is right and the enableUserMoves is true
        if (moveInfo != null) {
          if (moveInfo.length == 3) {
            if (moveInfo[2] == widget.controller.gameLogic.turn &&
                widget.controller.enableUserMoves) {
              return true;
            } else {
              print(
                  'either is not the right turn to play or user moves are not enable');
              return false;
            }
          } else {
            print('[moveInfo] length is not 3');
            return false;
          }
        } else {
          return false;
        }
      }, onAccept: (List moveInfo) async {
        /// Here is accepted information about the move
        /// the square from where the move originated (moveInfo[0]),
        /// the type of piece (moveInfo[1]), and the color (moveInfo[2])

        /// Accept only if the move made is of the color of the move turn to play in the game
        if (moveInfo[2] == widget.controller.gameLogic.turn) {
          /// Check if is a promotion move: the piece is a pawn, was in the rank 2 or 7
          /// and went to 1 or 8 (depending on the color)
          if (moveInfo[1] == "P" &&
              ((moveInfo[0][1] == "7" &&
                      widget.squareName[1] == "8" &&
                      moveInfo[2] == chessColor.Color.WHITE) ||
                  (moveInfo[0][1] == "2" &&
                      widget.squareName[1] == "1" &&
                      moveInfo[2] == chessColor.Color.BLACK))) {
            /// get the promotion piece selected by the user from the dialog
            final val = await _promotionDialog(context);
            if (val == 'n' || val == 'q' || val == 'r' || val == 'b') {
              widget.controller.gameLogic.move({
                "from": moveInfo[0],
                "to": widget.squareName,
                "promotion": val
              });

              /// Promotion -> notify listeners
              widget.controller.onPromotion();
            } else
              print('error didnt selected a piece');
          }

          /// if is not a promotion just make the move
          else {
            widget.controller.gameLogic
                .move({"from": moveInfo[0], "to": widget.squareName});
          }
          widget.controller.refreshBoard();
          setState(() {});
          widget.controller.getSanHistory();
        } else
          print('not right turn to move');
      }),
    );
  }

  /// Show dialog when pawn reaches last square
  Future<String?> _promotionDialog(BuildContext context) async {
    return showDialog<String>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        final color = widget.controller.gameLogic.turn;
        return AlertDialog(
          title: Text('Choose promotion'),
          content: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              InkWell(
                child: color == chessColor.Color.WHITE
                    ? WhiteQueen()
                    : BlackQueen(),
                onTap: () {
                  Navigator.of(context).pop("q");
                },
              ),
              InkWell(
                child:
                    color == chessColor.Color.WHITE ? WhiteRook() : BlackRook(),
                onTap: () {
                  Navigator.of(context).pop("r");
                },
              ),
              InkWell(
                child: color == chessColor.Color.WHITE
                    ? WhiteBishop()
                    : BlackBishop(),
                onTap: () {
                  Navigator.of(context).pop("b");
                },
              ),
              InkWell(
                child: color == chessColor.Color.WHITE
                    ? WhiteKnight()
                    : BlackKnight(),
                onTap: () {
                  Navigator.of(context).pop("n");
                },
              ),
            ],
          ),
        );
      },
    ).then((value) {
      return value;
    });
  }

  /// Get image to display on square
  Widget _getImageToDisplay(
      {required double size, required ChessboardController game}) {
    Widget imageToDisplay = const SizedBox();

    if (game.gameLogic.get(widget.squareName) == null) {
      const SizedBox();
    }

    String piece =
        (game.gameLogic.get(widget.squareName)!.color == chessColor.Color.WHITE
                ? 'W'
                : 'B') +
            game.gameLogic.get(widget.squareName)!.type.toUpperCase();

    switch (piece) {
      case "WP":
        imageToDisplay = WhitePawn(size: size);
        break;
      case "WR":
        imageToDisplay = WhiteRook(size: size);
        break;
      case "WN":
        imageToDisplay = WhiteKnight(size: size);
        break;
      case "WB":
        imageToDisplay = WhiteBishop(size: size);
        break;
      case "WQ":
        imageToDisplay = WhiteQueen(size: size);
        break;
      case "WK":
        imageToDisplay = WhiteKing(size: size);
        break;
      case "BP":
        imageToDisplay = BlackPawn(size: size);
        break;
      case "BR":
        imageToDisplay = BlackRook(size: size);
        break;
      case "BN":
        imageToDisplay = BlackKnight(size: size);
        break;
      case "BB":
        imageToDisplay = BlackBishop(size: size);
        break;
      case "BQ":
        imageToDisplay = BlackQueen(size: size);
        break;
      case "BK":
        imageToDisplay = BlackKing(size: size);
        break;
      default:
        imageToDisplay = const SizedBox();
    }

    return imageToDisplay;
  }
}
